/* $Id: dirname.c,v 1.2 2020-11-06 17:55:54 phil Exp $
 * from FreeBSD 12
 * hacked for Windoze by Phil Budne
 */
/*-
 * Copyright (c) 2015-2016 Nuxi, https://nuxi.nl/
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
#include <sys/cdefs.h>
__FBSDID("$FreeBSD$");

#include <libgen.h>
*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H defined */

#include <stdio.h>			/* for lib.h */
#include <ctype.h>			/* isalpha */

#include "str.h"
#include "h.h"
#include "snotypes.h"
#include "lib.h"

#define ISSLASH(C) ((C) == '/' || (C) == '\\')

char *
dirname(char *path)
{
	char *end;
	char *pp = path;

	/*
	 * If path is a null pointer or points to an empty string,
	 * dirname() shall return a pointer to the string ".".
	 */
	if (path == NULL || *path == '\0')
		return (".");

	if (isalpha(*pp) && pp[1] == ':')
	    pp += 2;

#if 0
	if (ISSLASH(*pp))
	    pp++;
#endif

	/* Find end of last pathname component. */
	end = pp + strlen(pp);
	while (end > pp + 1 && ISSLASH(*(end - 1)))
		--end;

	/* Strip off the last pathname component. */
	while (end > pp && !ISSLASH(*(end - 1)))
		--end;

	/*
	 * If path does not contain a '/', then dirname() shall return a
	 * pointer to the string ".".
	 */
	if (end == pp) {
		pp[0] = '.';
		pp[1] = '\0';
		return (path);		/* PLB ?? */
	}

	/*
	 * Remove trailing slashes from the resulting directory name. Ensure
	 * that at least one character remains.
	 */
	while (end > pp + 1 && ISSLASH(*(end - 1)))
		--end;

	/* Null terminate directory name and return it. */
	*end = '\0';
	return (path);
}

#ifdef MAIN
#include <stdlib.h>			/* free */

void
t(char *s) {
    char *d = strdup(s);
    printf("%s => %s\n", s, dirname(d));
    free(d);
}

int
main() {
    t("/a/b/c");
    t("\\a\\b\\c");

    t("/a/b");
    t("\\a\\b");

    t("/a");
    t("\\a");

    t("a");

    t("q:/a/b/c");
    t("q:\\a\\b\\c");

    t("q:/a/b");
    t("q:\\a\\b");

    t("q:/a");
    t("q:\\a");
    t("q:a");

    return 0;
}
#endif
